;;;_ tester/launch/rtest.el --- Tests of tester/launch.el

;;;_. Headers
;;;_ , License
;; Copyright (C) 2010  Tom Breton (Tehom)

;; Author: Tom Breton (Tehom) <tehom@panix.com>
;; Keywords: lisp, maint, internal

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;;_ , Commentary:

;; 


;;;_ , Requires

(require 'rtest-define)

(require 'emtest/runner/launch)
(require 'emtest/runner/launch/testhelp)

(require 'emtest/runner/define)
(require 'emtest/runner/tester)
(require 'emtest/testhelp/tagnames)
(require 'emtest/testhelp/mocks/filebuf)
(require 'emtest/testhelp/deep-type-checker)
(require 'emtest/testhelp/testpoint)
(require 'emtest/testhelp/standard)
(require 'emtest/testhelp/misc) ;;For emth:all-different

;;;_. Body

;;;_  . Tests

(rtest:deftest emt:sexp-at-point

   ;;This can only be tested fully-manually.
   '
   (  "Situation: WRITEME.
Response: WRITEME."
      (emt:sexp-at-point
	 '("Situation: While executing a dummy test with no checks"
	     (progn))))

   '  
   (  "Situation: WRITEME.
Response: WRITEME."
      (emt:sexp-at-point
	 '("Situation: While executing a dummy test
with a successful check.   The test is fully written out." 
	     (emth:should-f t))))
   
   '
   (  "Situation: WRITEME.
Response: WRITEME."
      (emt:sexp-at-point
	 '("Situation: While executing a dummy test with a successful
check.  The test is a macro.
"
	     (should t))))

   '
   (  "Situation: WRITEME.
Response: WRITEME."
      (emt:sexp-at-point
	 '("Situation: While executing a dummy test with a failing check"
	     (should nil))))
   '
   (  "Situation: Body raises an error of type `emt:already-handled'.
Response: Reports a failure but essentially nothing special is reported."
      (emt:sexp-at-point
	 '("Situation: While executing a dummy test that raises
`emt:already-handled'."
	     (signal 'emt:already-handled ())))))

;;;_  . Test helper emtt:suite:th:get-id-list

(defun emtt:suite:th:get-id-list (sym) 
   ""
   (let
      ((test-id-list ()))
      (emtp:eval
	 (emtt:th:run-suite sym #'ignore)
	 (tp*
	    (:id tp:a084136e-8f02-49a5-ac0d-9f65509cedf2 
	       :count nil 
	       :fallthru t)
	    (test-id)
	    (typecase test-id
	       ;;Intercept clause
	       (emthow:indexed-clause
		  (push test-id test-id-list)
		  ;;Don't try to run it, return instead.
		  (throw 'emtp:tag-return nil))
	       ;;Let suite fall thru to handler
	       (emthow:suite t)
	       ;;We don't expect to see any other types of
	       ;;explores.
	       (t
		  (error "This test shouldn't reach here")))))
      test-id-list))

;;;_  . Tests
(rtest:deftest emtt:suite

   (  "Situation: SUITE-SYM has a test suite defined.
Response: That test-suite is run.  A result is returned."
      (emth:let-noprops '(dummy-sym)
	 (emt:deftest-3 dummy-sym
	    (nil (progn (emt:doc "Docstring"))))
	 (emtp:eval
	    (emtt:th:run-suite 'dummy-sym 
	       #'(lambda (x)
		    (emty:check x emt:testral:report)))
	    (tp-reached tp:798212b4-1abe-4779-beb1-baf53ff39a8c 
	       1)
	    (tp* 
	       (  :id tp:a084136e-8f02-49a5-ac0d-9f65509cedf2 
		  :count nil
		  :fallthru t)
	       (test-id)
	       (typecase test-id
		  ;;Intercept clause, count it
		  (emthow:indexed-clause
		     ;;NOT emthow:form
		     ;;A reached-point for counting invocations.
		     (emtp tp:798212b4-1abe-4779-beb1-baf53ff39a8c
			())
		     ;;Fall thru
		     t)
		  ;;Let suite fall thru to handler
		  (emthow:suite t)
		  ;;We don't expect to see any other types of
		  ;;explores.
		  (t
		     (error "This test shouldn't reach here")))))
	 t))
   
   
   (  "Shows: Exactly the clauses of suite are run.
Situation: Suite has two clauses defined.  
Full exploration is used (Meaningless for now)
Response: Just two clauses are run.
Those suites have distinct IDs."
      (emth:let-noprops '(dummy-sym)
	 (emt:deftest-3 dummy-sym
	    (() (progn (emt:doc "Clause 1")))
	    (() (progn (emt:doc "Clause 2"))))
	 (let
	    ((all-test-ids))
	    (emtp:eval
	       (emtt:th:run-suite 'dummy-sym #'ignore)
	       (tp* 
		  (
		     :id tp:798212b4-1abe-4779-beb1-baf53ff39a8c 
		     :count 2
		     :fallthru t)
		  (test-id)
		  (push test-id all-test-ids))
	       (tp* 
		  (  :id tp:a084136e-8f02-49a5-ac0d-9f65509cedf2 
		     :count nil
		     :fallthru t)
		  (test-id)
		  (typecase test-id
		     ;;Intercept clause, count it
		     (emthow:indexed-clause
			;;NOT emthow:form
			(emtp tp:798212b4-1abe-4779-beb1-baf53ff39a8c
			   (test-id))
			;;Don't try to run it, return instead.
			(throw 'emtp:tag-return nil))
		     ;;Let suite fall thru to handler
		     (emthow:suite t)
		     ;;We don't expect to see any other types of
		     ;;explores.
		     (t
			(error "This test shouldn't reach here")))))
	    (assert
	       (emth:all-different all-test-ids)
	       t))
	 t))
   

   (  "Situation: A clause has changed within a suite.
Operation: That test suite is run.
Behavior: The clause still gets the same test-ID as before."
      (emth:let-noprops '(dummy-sym)
	 (emt:deftest-3 dummy-sym
	    (() (progn (emt:doc "Clause 1") (original-form))))

	 (let*
	    (  (old-test-id-list
		  (emtt:suite:th:get-id-list 'dummy-sym))
	       (old-test-id (car old-test-id-list)))
	    (assert (= (length old-test-id-list) 1) t)
	       
	    (emt:deftest-3 dummy-sym
	       (() (progn (emt:doc "Clause 1") (new-form))))

	       
	    (let*
	       (  (new-test-id-list
		     (emtt:suite:th:get-id-list 'dummy-sym))
		  (new-test-id (car new-test-id-list)))
	       (assert (= (length new-test-id-list) 1) t)

	       (assert 
		  (equal old-test-id new-test-id)
		  t)))
	 t)))

;;;_  . emtt:defun-at-point
;;emtt:defun-at-point is direct, it just joins other functions

;;;_   , emtel:suite-sym-at-point

(emt:deftest-3 emtel:suite-sym-at-point

   ;;Need example text, presumably from one or more example files.
   ;;They are in "t/examples/editor/find-names/1.el"

   ;;To set up point - mockbuf should provide mark conversion but it
   ;;doesn't yet.

   ;;Test situations: In a test definition.  In a function definition.
   ;;Between them.  After a definition (find that, not next).

   ;;So foo should be found for:
   ;;!Before all foo definitions.
   ;;!In function.
   ;;!Between function and test.
   ;;!In test definition.
   ;;!In test clause.

   ;;Something else or nothing (don't care) should be found for "After
   ;;all foo definitions" (not (eq (emtel:suite-sym-at-point) 'foo))


   ;;Nothing should be found in empty file (Make from string)
   (()
      (emtb:with-buf 
	 (:string "\n\n\n\n")
	 (emt:doc "Situation: In empty file.")
	 (emt:doc "Response: Return `nil'.")
	 (emacs-lisp-mode)
	 (should
	    (null 
	       (emtel:suite-sym-at-point))))))


;;;_  . Tests
(rtest:deftest emt:library

   (  "Situation: In the known (count 2) context of libraries and
suites.
Full exploration is used (Meaningless for now)
Operation: We launch a library
Response: The suite-handler part runs exactly twice
\(Not tested: Exactly those two suites are seen.)
\(Not tested: Those suites have distinct IDs.)
"
      (emt:library:th ((count 2))
	 (emtp:eval
	    (emt:library
	       (emtg (type sym))
	       ;;$$Library does not return a result object yet.  Will
	       ;;type-check it.  But for now, we can't pass any
	       ;;argument. 
	       ;;(emty:check x emt:result-group)
	       #'ignore)
	       
	    (tp-reached tp:798212b4-1abe-4779-beb1-baf53ff39a8c 
	       (emtg (type count)))
	    (tp* 
	       (  :id tp:a084136e-8f02-49a5-ac0d-9f65509cedf2 
		  :count nil
		  :fallthru t)
	       (test-id)
	       (typecase test-id
		  ;;Intercept suite
		  (emthow:suite
		     ;;A reached-point for counting invocations.
		     (emtp tp:798212b4-1abe-4779-beb1-baf53ff39a8c
			())
		     ;;Don't try to explore its clauses, return
		     ;;instead.
		     (throw 'emtp:tag-return nil))
		  ;;Let library fall thru to handler
		  (emthow:library:elisp-load t)
		  ;;We don't expect to see any other types of
		  ;;explores.
		  (t
		     (error "This test shouldn't reach here"))))))))

;;;_. Footers
;;;_ , Provides

(provide 'emtest/runner/launch/rtest)

;;;_ * Local emacs vars.
;;;_  + Local variables:
;;;_  + mode: allout
;;;_  + End:

;;;_ , End
;;; tester/launch/rtest.el ends here
