;;;_ tinydb/persist.el --- Create persisting data

;;;_. Headers
;;;_ , License
;; Copyright (C) 2007,2010  Tom Breton (Tehom)

;; Author: Tom Breton (Tehom) <tehom@panix.com>
;; Keywords: 

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;;_ , Commentary:

;; This was originally part of my `password' library.  I separated it
;; so that it could be re-used.

;;Drawbacks: It only makes one object persist per buffer.  This could
;;be remedied with a major rewrite.  Even so, it would probably be
;;less safe.

;;;_ , Requires
(require 'pp)
(require 'tinydb/asynq)


;;;_. Code:

;;;_ , Internal:

;;;_  . tehom-persist-sync-obj-to-buffer
;;$$OBSOLESCENT.
;;Internal only
(defun tehom-persist-sync-obj-to-buffer (initial-obj &optional type-pred)
   "Return the current buffer's data as an object.
If we can't read an object, signal error.
If TYPE-PRED is given and object doesn't satisfy it, signal error.

Do NOT call this unless you know what you are doing.  It must only be
called in a buffer visiting a file meant to store sexps persistently."

   (let
      ((obj 
	  (condition-case err
	     (if
		(= (buffer-size) 0)
		initial-obj
		(progn
		   (goto-char (point-min))
		   (read (current-buffer))))
	     (error 
		(error
		   "In persist buffer %s, contents could not be read"
		   (current-buffer))
		'()))))

      (if
	 (or (null type-pred) (funcall type-pred obj))
	 obj
	 (progn
	    (error
	       "In persist buffer %s, object was wrong type"
	       (current-buffer))
	    '()))))


;;$$OBSOLESCENT.
(defalias 'tehom-persist-sync-list-to-buffer
   'tehom-persist-sync-obj-to-buffer) 

;;;_  . tehom-persist-sync-file-to-obj

;;$$OBSOLESCENT.  Use `tinydb-persist--write-obj'
;;Internal only
(defun tehom-persist-sync-file-to-obj (obj &optional force-save type-pred)
   "Sync to the current buffer and file to OBJ

Do NOT call this unless you know what you are doing.  It will ERASE
the contents of the buffer and write another object into it."

   (unless
      (or
	 (not type-pred)
	 (funcall type-pred obj))
      (error "Object %s does not satisfy %s" obj type-pred))

   (let
      ((old-str (buffer-string)))
      (erase-buffer)
      (insert (pp-to-string obj))
      (condition-case err
	 (progn
	    (goto-char (point-min))
	    (read (current-buffer)))
	 (error
	    (erase-buffer)
	    (insert old-str)
	    (apply #'signal (car err) (cdr err)))))
   

   (when force-save
      (let
	 ((file-precious-flag t))
	 ;;Save, forcing backups to exist.
	 (save-buffer 64))))

;;$$OBSOLESCENT.
(defalias 'tehom-persist-sync-file-to-list 'tehom-persist-sync-file-to-obj)
;;;_  . tinydb-persist--write-obj
(defun tinydb-persist--write-obj (obj &optional force-save)
   ""

   (let
      ((old-str (buffer-string)))
      (erase-buffer)
      (insert (pp-to-string obj))
      ;;If it can't be read back, it's an error
      (condition-case err
	 (progn
	    (goto-char (point-min))
	    (read (current-buffer)))
	 (error
	    (erase-buffer)
	    (insert old-str)
	    (apply #'signal (car err) (cdr err)))))
   
   (when force-save
      ;;Save, forcing backups to exist.
      (let
	 ((file-precious-flag t))
	 (save-buffer 64))))

;;;_ , Macros to use the persisting object

;;;_  . tehom-persist-buffer-as-const-obj
;;$$OBSOLESCENT.
(defmacro tehom-persist-buffer-as-const-obj 
   (persist-file var initial-obj type-pred &rest body)
   "Treat the password buffer as an immutable object named VAR"
   
   `(with-current-buffer (find-file-noselect ,persist-file)
       (let
	  ((,var 
	      (tehom-persist-sync-obj-to-buffer ,initial-obj ,type-pred)))
	  ,@body)))

;;;_  . tehom-persist-buffer-as-const-list
;;$$OBSOLESCENT.
(defmacro tehom-persist-buffer-as-const-list (persist-file var &rest body)
   "Treat the password buffer as an immutable list named VAR"
   `(tehom-persist-buffer-as-const-obj
       ,persist-file ,var () #'listp ,@body))


;;;_  . tehom-persist-buffer-as-mutable-obj
;;$$OBSOLESCENT.
(defmacro tehom-persist-buffer-as-mutable-obj 
   (persist-file var initial-obj type-pred &rest body)
   ""
   `
   (with-current-buffer (find-file-noselect ,persist-file)
       (let
	  ((,var (tehom-persist-sync-obj-to-buffer ,initial-obj ,type-pred)))
	  (prog1
	     (progn ,@body)
	     (tehom-persist-sync-file-to-obj ,var t ,type-pred)))))


;;;_  . tehom-persist-buffer-as-mutable-list

;;$$OBSOLESCENT.
(defmacro tehom-persist-buffer-as-mutable-list (persist-file var &rest body)
   "Treat the file PERSIST-FILE as a mutable object named by the
symbol VAR."
   `(tehom-persist-buffer-as-mutable-obj
       ,persist-file ,var () #'listp ,@body))

;;;_  . tehom-update-persist-buffer

;;$$OBSOLESCENT.
(defun tehom-update-persist-buffer 
   (persist-file obj &optional force-save type-pred)
   "Assign OBJ to the persisting object in the buffer visiting PERSIST-FILE.
If TYPE-PRED is given, only write OBJ if it satisfies TYPE-PRED,
otherwise signal error."

   (with-current-buffer (find-file-noselect persist-file)
      (tehom-persist-sync-file-to-obj obj force-save type-pred)))


;;;_ , File-buffer instantiation of asynq

;;;_  . tinydb-persist-make-q
(defun tinydb-persist-make-q 
   (filename initial-obj &optional eager-save type-pred)
   "Make a file-based persisting queue"
   (tinydb-make-q
      ;;This is the setup for file-based asynq
      #'(lambda (filename initial-object eager-save)
	   (with-current-buffer
	      (find-file-noselect filename eager-save)
	      (declare (special persist-eager-save))
	      (set 
		 (make-local-variable
		    'persist-eager-save) 
		 eager-save)
	      (when
		 (= (buffer-size) 0)
		 ;;$$USE ME INSTEAD
		 '(tinydb-persist--write-obj initial-object eager-save)
		 (tehom-persist-sync-file-to-obj
		    initial-object eager-save))
	      (current-buffer)))
		   
      ;;get
      #'(lambda (buffer)
	   (with-current-buffer buffer
	      ;;$$USE ME INSTEAD after testing.
	      '
	      (condition-case err
		 (progn
		    (goto-char (point-min))
		    (read (current-buffer)))
		 (error 
		    (error
		       "In persist buffer %s, contents could not be read"
		       (current-buffer))
		    '()))
	      (tehom-persist-sync-obj-to-buffer nil)))
		   
      ;;put
      #'(lambda (buffer obj)
	   (with-current-buffer buffer
	      (declare (special persist-eager-save))
	      ;;Type check is handled by asynq now
	      ;;$$USE ME INSTEAD
	      '(tinydb-persist--write-obj obj persist-eager-save)
	      (tehom-persist-sync-file-to-obj 
		 obj 
		 persist-eager-save)
	      buffer))
      type-pred
      filename initial-obj eager-save))



;;;_: Footers

;;;_ * Local emacs vars.
;;;_  + Local variables:
;;;_  + mode: allout
;;;_  + End:

(provide 'tinydb/persist)
;;; tinydb/persist.el ends here
